////////////////////////////////////////////////////////////////////////////////
//                                  Include                                   //
////////////////////////////////////////////////////////////////////////////////


#include "Oracle.h"
#include "Solver.h"
#include "Bundle.h"
#include "OPTtypes.h"
#include <fstream>
using namespace Bundle_di_unipi_it;
using namespace OPTtypes_di_unipi_it;
using namespace std;


////////////////////////////////////////////////////////////////////////////////
//                                   Main                                     //
////////////////////////////////////////////////////////////////////////////////


int main(int argc, char* argv[])
{
  //----- Create --------------------------------------------------------------1

  // Create oracle object
  Oracle *oracle = new Oracle(argv[1]);

  // Variables
  int i, j, k;
  long uSize = oracle->getUSize();
  long xSize = 0;
  long xDims = oracle->getXDims();
  if (xDims == 1)
    xSize = oracle->getXDim1Size();
  else if (xDims == 2)
    xSize = oracle->getXDim1Size() * oracle->getXDim2Size();

  // Create xMatrix
  ifstream param2("Parameters");
  Index maxBundleSize = 0;
  for (i = 0; i < 3; i++)
    DfltdSfInpt(&param2, maxBundleSize, Index(100));
  double** xMatrix = new (double* [maxBundleSize]);
  for (i = 0; i < maxBundleSize; i++)
    xMatrix[i] = new double[xSize];
  for (i = 0; i < maxBundleSize; i++)
    for (j = 0; j < xSize; j++)
      xMatrix[i][j] = 0;

  // Create solver object
  ifstream param("Parameters");
  Bundle *b = new Solver(&param, oracle, uSize, xMatrix);

  // Set unconstrained multipliers
  b->SetUC(0, 0, oracle->getUCVector());

  // If the log has to be done, it is done on clog
  #if (LOG_BND)
  b->SetBLog(&clog, 0);
  #endif

  //---------------------------------------------------------------------------1

  //----- Solve ---------------------------------------------------------------2

  // Print Iterations
  cout << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << "--                             Iterations:                            --" << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << endl;

  if (b->Solve() == Bundle::kOK) {

    // Print Solution
    cout << endl;

    cout << "------------------------------------------------------------------------" << endl;
    cout << "--                              Solution:                             --" << endl;
    cout << "------------------------------------------------------------------------" << endl;

    cout << endl;

    if (oracle->getIsMinProblem())
      cout << "  f = " << -b->ReadBestFiVal() << endl;
    else
      cout << "  f = " << b->ReadBestFiVal() << endl;

    cout << endl;

    ofstream ufile("uVector");
    double *u = new double[uSize];
    b->ReadSol(u);
    for (i = 0; i < uSize; i++)
      ufile << "  u" << i + 1 << " = " << u[i] << endl;
    cout << "  u-vector written to file: uVector" << endl;
    delete u;

    cout << endl;

    ofstream xfile("xVector");
    double x;
    if (xDims == 1) {
      for (j = 0; j < xSize; j++) {
        x = 0;
        for (i = 0; i < b->ReadBDim(); i++)
          x += xMatrix[b->ReadBase()[i]][j] * b->ReadMult()[i];
        xfile << "  x" << j + 1 << " = " << x << endl;
      }
    } else if (xDims == 2) {
      long xDim1Size = oracle->getXDim1Size();
      long xDim2Size = oracle->getXDim2Size();
      for (j = 0; j < xDim1Size; j++)
        for (k = 0; k < xDim2Size; k++) {
          x = 0;
          for (i = 0; i < b->ReadBDim(); i++)
            x += xMatrix[b->ReadBase()[i]][j * xDim1Size + k] * b->ReadMult()[i];
	  xfile << "  x" << j + 1 << "," << k + 1 << " = " << x << endl;
	}
    }
    cout << "  x-vector written to file: xVector" << endl;

    cout << endl;
    cout << "------------------------------------------------------------------------" << endl;
    cout << endl;
  }

  //---------------------------------------------------------------------------2

  //----- Delete --------------------------------------------------------------3

  // Delete solver object
  delete b;

  // Delete xMatrix
  for (int i = 0; i < maxBundleSize; i++)
    delete xMatrix[i];
  delete xMatrix;

  // Delete oracle object
  delete oracle;

  //---------------------------------------------------------------------------3

  return(0);
}


////////////////////////////////////////////////////////////////////////////////
