////////////////////////////////////////////////////////////////////////////////
//                                  Include                                   //
////////////////////////////////////////////////////////////////////////////////


#include "Solver.h"
#include "OPTvect.h"
using namespace Bundle_di_unipi_it;


////////////////////////////////////////////////////////////////////////////////
//                               Public: Solver                               //
////////////////////////////////////////////////////////////////////////////////


Solver::Solver(istream *param, Oracle *aOracle, cIndex uDimSize,
                   double **axMatrix)
          : QPBundle(param, uDimSize)
{
  // Set oracle pointer
  oracle = aOracle;

  // Create vector of multipliers
  uVector = new double[uDimSize];
  uSize = uDimSize;

  // Set xMatrix pointer
  xMatrix = axMatrix;
  xPos = 0;

  // Initiate subgradient vector to null
  SG = NULL;
}


////////////////////////////////////////////////////////////////////////////////
//                               Public: ~Solver                              //
////////////////////////////////////////////////////////////////////////////////


Solver::~Solver()
{
  // Delete vector of multipliers
  delete [] uVector;
}


////////////////////////////////////////////////////////////////////////////////
//                              Protected: SetGi                              //
////////////////////////////////////////////////////////////////////////////////


void Solver::SetGi(SgRow SubG, cIndex Name)
{
  // Point the subgradient vector to the right address
  SG = SubG;

  // Position of the new subgradient and x.
  xPos = Name;
}


////////////////////////////////////////////////////////////////////////////////
//                               Protected: Fi                                //
////////////////////////////////////////////////////////////////////////////////


HpNum Solver::Fi(cLMRow Lam1, cIndex_Set LamB, Index LamBd)
{
  // Initiate
  double fValue = 0;

  // Fill uVector
  if (LamB)
  {
    for (Index i = 0; i < uSize; i++)
    {
      if (i == *LamB) {
        uVector[i] = *(Lam1++);
        LamB++;
      } else
        uVector[i] = 0;
    }
  }
  else
  {
    for (Index i = 0; i < uSize; i++)
      uVector[i] = *(Lam1++);
  }

  // Oracle: (In: u vector) (Out: function value, subgradient vector, x vector)
  oracle->Solve(uVector, fValue, SG, xMatrix[xPos]);

  // Return function value
  return (fValue);
}


////////////////////////////////////////////////////////////////////////////////
//                             Protected: GetGi                               //
////////////////////////////////////////////////////////////////////////////////


SIndex Solver::GetGi(HpNum &Eps1, cIndex_Set &SGBse)
{
  Eps1 = 0;

  SGBse = NULL;

  return (0);
}


////////////////////////////////////////////////////////////////////////////////
